@@ -5,17 +5,7 @@ from copy import deepcopy
 from dataclasses import dataclass
 from datetime import timedelta
 from itertools import product
-from typing import (
-    Any,
-    Callable,
-    Generic,
-    Optional,
-    TypeVar,
-    Dict,
-    List,
-    overload,
-    Sequence,
-)
+from typing import Any, Callable, Dict, Generic, List, Optional, Sequence, TypeVar
 from unittest import TestCase, TestSuite
 from unittest.mock import patch
 
@@ -33,6 +23,9 @@ AGA_RESERVED_KEYWORDS = {
     "aga_name",
     "aga_weight",
     "aga_value",
+    "aga_extra_credit",
+    "aga_override_check",
+    "aga_override_test",
 }
 
 
@@ -148,7 +141,12 @@ class _TestInputs(TestCase, Generic[Output]):
         aga_name: Optional[str],
         aga_weight: int,
         aga_value: float,
+        aga_extra_credit: float,
         aga_mock_input: bool,
+        aga_override_check: Optional[Callable[[TestCase, Output, Output], None]],
+        aga_override_test: Optional[
+            Callable[[TestCase, Callable[..., Output], Callable[..., Output]], None]
+        ],
         **kwargs: Any,
     ) -> None:
         super().__init__()
@@ -157,7 +155,9 @@ class _TestInputs(TestCase, Generic[Output]):
         self._mock_input = aga_mock_input
         self._expect = aga_expect
         self._expect_stdout = aga_expect_stdout
-        self.score_info = ScoreInfo(aga_weight, aga_value)
+        self._override_check = aga_override_check
+        self._override_test = aga_override_test
+        self.score_info = ScoreInfo(aga_weight, aga_value, aga_extra_credit)
 
         self._args = args
         self._kwargs = kwargs
@@ -186,6 +186,14 @@ class _TestInputs(TestCase, Generic[Output]):
         if they differ. This means it can be plugged into any unittest TestCase as a
         helper method.
         """
+        if self._override_test:
+            self._override_test(self, golden, under_test)
+            return
+
+        if self._override_check:
+            self._override_check(self, self._eval(golden), self._eval(under_test))
+            return
+
         if metadata.check_stdout:
             golden_stdout, golden_output = self._eval(with_captured_stdout(golden))
             under_test_stdout, under_test_output = self._eval(
@@ -219,7 +227,13 @@ class _TestInputs(TestCase, Generic[Output]):
             diff_explanation = ""
             diff = ""
 
-        self.assertEqual(
+        if isinstance(expected, float) and isinstance(got, (float, int)):
+            comparator = self.assertAlmostEqual  # type: ignore
+
+        else:
+            comparator = self.assertEqual  # type: ignore
+
+        comparator(  # type: ignore
             got,
             expected,
             msg=msg_format.format(
@@ -297,10 +311,12 @@ class _TestInputs(TestCase, Generic[Output]):
 class _TestInputGroup(Generic[Output]):
     """A group of test cases with shared configuration."""
 
-    def __init__(self, weight: int = 1, value: float = 0.0) -> None:
+    def __init__(
+        self, weight: int = 1, value: float = 0.0, extra_credit: float = 0.0
+    ) -> None:
         self._test_cases: list[_TestInputs[Output]] = []
         self._prizes: list[Prize] = []
-        self.score_info = ScoreInfo(weight, value)
+        self.score_info = ScoreInfo(weight, value, extra_credit)
 
     def add_test_case(self, case: _TestInputs[Output]) -> None:
         """Add a test case to the group."""
@@ -369,7 +385,12 @@ class Problem(Generic[Output]):
         aga_name: Optional[str] = None,
         aga_weight: int = 1,
         aga_value: float = 0.0,
+        aga_extra_credit: float = 0.0,
         aga_expect_stdout: Optional[str | Sequence[str]] = None,
+        aga_override_check: Optional[Callable[[TestCase, Output, Output], None]] = None,
+        aga_override_test: Optional[
+            Callable[[TestCase, Callable[..., Output], Callable[..., Output]], None]
+        ] = None,
         **kwargs: Any,
     ) -> None:
         """Add a test case to the current group.
@@ -385,6 +406,9 @@ class Problem(Generic[Output]):
             aga_name=aga_name,
             aga_weight=aga_weight,
             aga_value=aga_value,
+            aga_extra_credit=aga_extra_credit,
+            aga_override_check=aga_override_check,
+            aga_override_test=aga_override_test,
             aga_mock_input=self._config.problem.mock_input,
             **kwargs,
         )
@@ -567,27 +591,6 @@ def _check_reserved_keyword(kwd: str) -> None:
         )
 
 
-@overload
-def test_case(
-    *args: Any,
-    aga_expect: Optional[Any] = ...,
-    aga_hidden: bool = ...,
-    aga_name: Optional[str] = ...,
-    aga_weight: int = ...,
-    aga_value: float = ...,
-    aga_expect_stdout: Optional[str | Sequence[str]] = ...,
-    **kwargs: Any,
-) -> Callable[[Problem[Output]], Problem[Output]]:
-    ...
-
-
-@overload
-def test_case(
-    *args: Any, **kwargs: Any
-) -> Callable[[Problem[Output]], Problem[Output]]:
-    ...
-
-
 def test_case(
     *args: Any,
     **kwargs: Any,
@@ -621,6 +624,15 @@ def test_case(
         Score` for details.
     aga_value : float
         The test case's absolute score. See :ref:`Determining Score` for details.
+    aga_extra_credit : float
+        The test case's absolute extra credit score. See :ref:`Determining Score` for
+        details.
+    aga_override_check : Optional[Callable[[TestCase, Output, T], None]]
+        A function which overrides the equality assertions made by the library. See
+        :ref:`Overriding the Equality Check` for more.
+    aga_override_test : Optional[Callable[[TestCase, Callable[T], Callable[T]], None]]
+        A function which overrides the entire test behavior of the library. See
+        :ref:`Overriding the Entire Test` for more.
     kwargs :
         Keyword arguments to be passed to the functions under test. Any keyword starting
         with aga\_ is reserved.
@@ -643,31 +655,6 @@ def test_case(
     return outer
 
 
-@overload
-def test_cases(
-    *args: Iterable[Any],
-    aga_product: bool = True,
-    **kwargs: Iterable[Any] | Any,
-) -> Callable[[Problem[Output]], Problem[Output]]:
-    ...
-
-
-@overload
-def test_cases(
-    *args: Iterable[Any],
-    aga_expect: Iterable[Optional[Any]] | Optional[Any] = ...,
-    aga_hidden: Iterable[bool] | bool = ...,
-    aga_name: Iterable[Optional[str]] | Optional[str] = ...,
-    aga_weight: Iterable[int] | int = ...,
-    aga_value: Iterable[float] | float = ...,
-    aga_expect_stdout: Iterable[Optional[str | Sequence[str]]]
-    | Optional[str | Sequence[str]] = ...,
-    aga_product: bool = True,
-    **kwargs: Iterable[Any],
-) -> Callable[[Problem[Output]], Problem[Output]]:
-    ...
-
-
 def test_cases(
     *args: Iterable[Any],
     aga_product: bool = True,
@@ -782,7 +769,9 @@ def test_cases(
 
 
 def group(
-    weight: int = 1, value: float = 0.0
+    weight: int = 1,
+    value: float = 0.0,
+    extra_credit: float = 0.0,
 ) -> Callable[[Problem[Output]], Problem[Output]]:
     """Declare a group of problems.
 
@@ -801,7 +790,7 @@ def group(
     """
 
     def outer(prob: Problem[Output]) -> Problem[Output]:
-        prob.add_group(_TestInputGroup(weight, value))
+        prob.add_group(_TestInputGroup(weight, value, extra_credit))
         return prob
 
     return outer
